/**
 * @file table columns + actionColumns
 */
import { isBoolean, isFunction } from '@/utils/is';
import { cloneDeep } from 'lodash-es';
import type { Ref, ComputedRef } from 'vue';
import { ref, unref, computed, watch } from 'vue';
import { ACTION_COLUMN_FLAG, DEFAULT_ALIGN } from '../constant';
import { PaginationProps } from '../types/pagination';
import { BasicColumn, BasicTableProps } from '../types/table';

function handleActionColumn(propsRef: ComputedRef<BasicTableProps>, columns: BasicColumn[]): void {
	const actionColumn = unref(propsRef).actionColumn;
	if (!actionColumn) return;

	// 剔除掉columns里面的flag
	const otherAction = columns.find(column => column.flag === ACTION_COLUMN_FLAG);
	if (!otherAction) {
		columns.push({
			fixed: 'right',
			...actionColumn,
			flag: ACTION_COLUMN_FLAG,
		});
	}
}

// 设置是否自动省略
function handleItemOfColumns(item: BasicColumn, ellipsis: boolean) {
	const { key, dataIndex } = item;
	item.align = item.align || DEFAULT_ALIGN;
	if (ellipsis) {
		if (!key) {
			item.key = dataIndex;
		}
		if (!isBoolean(item.ellipsis)) {
			Object.assign(item, {
				ellipsis,
			});
		} else {
			item.ellipsis = ellipsis;
		}
	}
}

// 设置是否显示
function isShow(column: BasicColumn): boolean {
	let isShow = column.isShow ?? true;
	if (isBoolean(isShow)) {
		isShow = isShow;
	}
	if (isFunction(isShow)) {
		isShow = isShow(column);
	}
	return isShow;
}

export function useColumns(
	propsRef: ComputedRef<BasicTableProps>,
	paginationRef: ComputedRef<PaginationProps | boolean>,
) {
	const columnsRef = ref(unref(propsRef).columns) as unknown as Ref<BasicColumn[]>;
	let cacheColumns = unref(propsRef).columns;
	console.log(paginationRef);

	// 先处理columns
	const getColumnsRef = computed(() => {
		const columns = cloneDeep(unref(columnsRef));

		/**
		 * TODO: 表格头部增加序号
		 * showIndexColumn && isTreeTable,结合分页信息
		 */
		// handleIndexColumn(propsRef, paginationRef, columns)
		handleActionColumn(propsRef, columns);

		if (!columns) {
			return [];
		}

		// 超过宽度是否自动省略
		const { ellipsis } = unref(propsRef);

		columns.map(item => {
			// 但凡有以下2个，则无需设置ellipsis
			const { slots, customRender } = item;
			handleItemOfColumns(
				item,
				Reflect.has(item, 'ellipsis') ? !!item.ellipsis : !!ellipsis && !customRender && !slots,
			);
		});

		return columns;
	});

	// 筛选出需要展示的
	const getViewColumns = computed(() => {
		let viewColumns = unref(getColumnsRef);

		// TODO: 根据fixed: left | right 进行排序
		// viewColumns = sortFixedColumn(viewColumns)

		return viewColumns.filter(column => {
			// TODO: hasPremission
			return isShow(column);
		});
		// .map(column => {
		//     // TODO: 处理editCell, format,
		// 	const {slots} = column

		// 	// 处理slots:{title: xxx}的情况
		// 	if(!slots || !slots?.title) {

		// 	}
		// })
	});

	/**
	 * @description
	 * columnsRef的默认值为undefined，
	 * 当useTable中使用setProps后，propsRef的值更改，当columns有值，即不再是undefined，就会执行
	 * 所以无需判断 newColumns是否有值
	 */
	watch(
		() => unref(propsRef).columns,
		newColumns => {
			// props变化 -> getColumnsRef计算属性变化 -> getViewColumns计算属性变化
			columnsRef.value = newColumns;
			// TODO: 这一行目前还不知道干啥的
			cacheColumns = newColumns?.filter(item => !item.flag) ?? [];
		},
	);

	return {
		getColumnsRef,
		getViewColumns,
	};
}
