import type { WatchStopHandle } from 'vue';
import { watch, ref, unref, onUnmounted } from 'vue';
import { BasicTableProps } from '../types/table';
import { TableActionType } from '../types/tableAction';

export function useTable(tableProps: BasicTableProps) {
	const tableRef = ref<Nullable<TableActionType>>(null);
	const loadedRef = ref(false);
	let stopWatch: WatchStopHandle;

	function register(instance: TableActionType) {
		onUnmounted(() => {
			tableRef.value = null;
			loadedRef.value = false;
		});

		if (unref(loadedRef) && unref(tableRef) === instance) return;

		tableProps && instance.setProps(tableProps);
		tableRef.value = instance;
		loadedRef.value = true;

		stopWatch?.();

		stopWatch = watch(
			() => tableProps,
			() => {
				tableProps && instance.setProps(tableProps);
			},
			{
				immediate: true,
				deep: true,
			},
		);
	}

	function getInstance() {
		const tableInstance = unref(tableRef);
		if (!tableInstance) {
			throw new Error('请先注册table!');
		}
		return tableInstance;
	}

	const methods: TableActionType = {
		reload: async params => {
			getInstance().reload(params);
		},
		setProps: props => {
			getInstance().setProps(props);
		},
	};
	return [register, methods];
}
