import type { DefineComponent, Ref, Slots } from 'vue';
import { h, computed, unref } from 'vue';
import { isString } from '@/utils/is';
import TableHeader from '../components/TableHeader.vue';
import { BasicTableProps } from '../types/table';

export function useTableHeader(propsRef: Ref<BasicTableProps>, slots: Slots) {
	const getTableHeader = computed(() => {
		const { title, titleHelpMessage } = unref(propsRef);
		const hideTitle = !title && !slots.tableTitle;
		if (hideTitle && !isString(title)) {
			return {};
		}

		return {
			title: () => {
				h(
					TableHeader as unknown as DefineComponent,
					{ title, titleHelpMessage },
					{ ...(slots.tableTitle ? { tableTitle: slots.tableTitle } : {}) },
				);
			},
		};
	});

	return { getTableHeader };
}
