import { VNodeChild } from 'vue';
import type { ColumnProps } from 'ant-design-vue/lib/table/interface';
import { PaginationProps } from './pagination';

export interface BasicColumn extends ColumnProps {
	children: BasicColumn[];

	// 头部的提示文字
	helpMessage?: string | string[];

	flag?: 'INDEX' | 'DEFAULT' | 'CHECKBOX' | 'RADIO' | 'SWITCH' | 'ACTION';
	// 是否展示
	isShow?: boolean | ((column: BasicColumn) => boolean);
}

export interface FetchFieldSetting {
	// 请求接口当前页数字段
	pageField?: string;
	// 请求接口列表字段
	listField?: string;
	// 请求接口总页数字段
	pageSizeField?: string;
	// 请求接口总条数字段
	totalField?: string;
}

export interface FetchParams {
	page?: number | string;
	sortInfo?: Recordable;
	filterInfo?: Recordable;
}

export interface BasicTableProps {
	// 表格标题
	title?: string | JSX.Element | VNodeChild | ((title: Recordable) => string);
	// 表格help
	titleHelpMessage?: string | string[];
	columns: BasicColumn[];
	// 是否展示序号
	showIndexColumn?: boolean;
	// 请求接口
	api?: (...args: any) => Promise<any>;
	// 是否立即请求接口
	immediate?: boolean;
	// 接口配置字段
	fetchFieldsSetting?: Partial<FetchFieldSetting>;
	// 是否开启loading
	loading?: boolean;
	// table row-key
	rowKey?: string | ((record: Recordable) => string);
	// action
	actionColumn?: BasicColumn;
	// 数据
	dataSource?: Recordable[];
	// 是否为树型表格
	isTreeTable?: boolean;
	// 超过长度是否省略
	ellipsis?: boolean;
	// 分页
	pagination: PaginationProps | boolean;
}
