/**
 * @file 创建和使用app级别的provid&inject
 */
import { reactive, readonly as defineReadonly, provide, inject, InjectionKey, UnwrapRef } from 'vue';

type CreateContextOptions = {
	readonly?: boolean;
	useProvide?: boolean;
	// 是原始对象还是reactive对象
	native?: boolean;
};

type ShallowUnWrap<T> = {
	[P in keyof T]: UnwrapRef<T[P]>;
};

export function createContext<T>(key: InjectionKey<T> = Symbol(), context: any, opts: CreateContextOptions = {}) {
	const { readonly = true, useProvide = true, native = false } = opts;
	const state = reactive(context);
	const provideData = readonly ? defineReadonly(state) : state;
	useProvide && provide(key, native ? context : provideData);
	return { state };
}

export function useContext<T>(key: InjectionKey<T>, native?: boolean): T;
export function useContext<T>(key: InjectionKey<T>, defaultValue?: any, native?: boolean): T;

export function useContext<T>(key: InjectionKey<T> = Symbol(), defaultValue?: any): ShallowUnWrap<T> {
	return inject(key, defaultValue || {});
}
